@@ -5,11 +5,11 @@
 set -eu
 export VPREFIX="$1"
 
-# These values are filled automatically for git-archive(1) tarballs.
-# See: "export-subst" in gitattributes(5).
-distinfo_commit_full='cae7c45ea5a563c022001a54d1eee71c268f62b4'
-distinfo_commit_short='cae7c45e'
-distinfo_author_date='2019-09-29 16:51:55 +0100'
+# This value is filled automatically for git-archive(1) tarballs.
+# See also: "export-subst" in gitattributes(5).
+# shellcheck disable=SC2016
+distinfo_commit_short='6e37409d'
+
 if expr "$distinfo_commit_short" : '[0-9a-f]\{7,40\}$' >/dev/null; then
     echo "${VPREFIX}-g${distinfo_commit_short}-dist"
     exit 0
