@@ -3,8 +3,8 @@
 
 #include "util/string.h"
 
+void init_aliases(void);
 void add_alias(const char *name, const char *value);
-void sort_aliases(void);
 const char *find_alias(const char *name);
 void collect_aliases(const char *prefix);
 String dump_aliases(void);
