@@ -1,7 +1,7 @@
 #ifndef BIND_H
 #define BIND_H
 
-#include "command.h"
+#include "command/run.h"
 #include "terminal/key.h"
 #include "util/string.h"
 
@@ -17,6 +17,7 @@ void add_binding(const char *keystr, const char *command);
 void remove_binding(const char *keystr);
 const KeyBinding *lookup_binding(KeyCode key);
 void handle_binding(KeyCode key);
+void collect_bound_keys(const char *keystr_prefix);
 String dump_bindings(void);
 
 #endif
