@@ -2,14 +2,14 @@
 #define CMDLINE_H
 
 #include <sys/types.h>
+#include "history.h"
 #include "terminal/key.h"
-#include "util/ptr-array.h"
 #include "util/string.h"
 
 typedef struct {
     String buf;
     size_t pos;
-    ssize_t search_pos;
+    const HistoryEntry *search_pos;
     char *search_text;
 } CommandLine;
 
@@ -21,11 +21,6 @@ typedef enum {
 
 void cmdline_clear(CommandLine *c);
 void cmdline_set_text(CommandLine *c, const char *text);
-
-CommandLineResult cmdline_handle_key (
-    CommandLine *c,
-    PointerArray *history,
-    KeyCode key
-);
+CommandLineResult cmdline_handle_key(CommandLine *c, History *hist, KeyCode key);
 
 #endif
