@@ -9,7 +9,7 @@ void select_block(void);
 void unselect(void);
 void cut(size_t len, bool is_lines);
 void copy(size_t len, bool is_lines);
-void insert_text(const char *text, size_t size);
+void insert_text(const char *text, size_t size, bool move_after);
 void paste(bool at_cursor);
 void delete_ch(void);
 void erase(void);
@@ -17,8 +17,9 @@ void insert_ch(CodePoint ch);
 void join_lines(void);
 void shift_lines(int count);
 void clear_lines(void);
+void delete_lines(void);
 void new_line(void);
 void format_paragraph(size_t text_width);
-void change_case(int mode);
+void change_case(char mode);
 
 #endif
