@@ -1,14 +1,16 @@
 #ifndef EDITOR_H
 #define EDITOR_H
 
+#include <signal.h>
 #include <stdbool.h>
+#include <stddef.h>
 #include "cmdline.h"
-#include "mode.h"
+#include "encoding.h"
+#include "history.h"
 #include "options.h"
-#include "encoding/encoding.h"
-#include "terminal/color.h"
 #include "util/macros.h"
 #include "util/ptr-array.h"
+#include "util/string-view.h"
 
 typedef enum {
     EDITOR_INITIALIZING,
@@ -20,28 +22,27 @@ typedef enum {
     INPUT_NORMAL,
     INPUT_COMMAND,
     INPUT_SEARCH,
-    INPUT_GIT_OPEN,
 } InputMode;
 
 typedef struct {
     EditorStatus status;
-    const EditorModeOps *mode_ops[4];
     InputMode input_mode;
     CommandLine cmdline;
     GlobalOptions options;
-    const char *home_dir;
+    StringView home_dir;
     const char *user_config_dir;
+    const char *xdg_runtime_dir;
     Encoding charset;
-    const char *pager;
     bool child_controls_terminal;
     bool everything_changed;
     bool term_utf8;
+    int exit_code;
     size_t cmdline_x;
-    PointerArray search_history;
-    PointerArray command_history;
+    PointerArray buffers;
+    History search_history;
+    History command_history;
     const char *const version;
-    void (*resize)(void);
-    void (*ui_end)(void);
+    volatile sig_atomic_t resized;
 } EditorState;
 
 extern EditorState editor;
@@ -57,12 +58,14 @@ static inline void set_input_mode(InputMode mode)
 }
 
 void init_editor_state(void);
-char *editor_file(const char *name) XSTRDUP;
-char get_confirmation(const char *choices, const char *format, ...) PRINTF(2);
+const char *editor_file(const char *name) NONNULL_ARGS_AND_RETURN;
+char status_prompt(const char *question, const char *choices) NONNULL_ARGS;
+char dialog_prompt(const char *question, const char *choices) NONNULL_ARGS;
 void any_key(void);
 void normal_update(void);
-void handle_sigwinch(int signum);
 void suspend(void);
 void main_loop(void);
+void ui_start(void);
+void ui_end(void);
 
 #endif
