@@ -3,6 +3,7 @@
 
 #include <stdbool.h>
 #include <stddef.h>
+#include "util/string-view.h"
 
 typedef struct {
     // Size in bytes
@@ -23,8 +24,8 @@ typedef struct {
 } IndentInfo;
 
 char *make_indent(size_t width);
-char *get_indent_for_next_line(const char *line, size_t len);
-void get_indent_info(const char *buf, size_t len, IndentInfo *info);
+char *get_indent_for_next_line(const StringView *line);
+void get_indent_info(const StringView *buf, IndentInfo *info);
 bool use_spaces_for_indent(void);
 size_t get_indent_level_bytes_left(void);
 size_t get_indent_level_bytes_right(void);
