@@ -1,7 +1,9 @@
 #ifndef LOCK_H
 #define LOCK_H
 
-int lock_file(const char *filename);
+#include <stdbool.h>
+
+bool lock_file(const char *filename);
 void unlock_file(const char *filename);
 
 #endif
