@@ -2,9 +2,8 @@
 #define SCREEN_H
 
 #include <stdbool.h>
-#include <stddef.h>
 #include "buffer.h"
-#include "terminal/color.h"
+#include "syntax/color.h"
 #include "view.h"
 #include "window.h"
 
@@ -15,12 +14,11 @@ void update_window_sizes(void);
 void update_line_numbers(Window *win, bool force);
 void update_screen_size(void);
 void set_color(const TermColor *color);
-void set_builtin_color(enum builtin_color c);
+void set_builtin_color(BuiltinColorEnum c);
 void mask_color(TermColor *color, const TermColor *over);
 
 // screen-cmdline.c
 void update_command_line(void);
-size_t print_command(char prefix);
 void show_message(const char *msg, bool is_error);
 
 // screen-tabbar.c
