@@ -3,11 +3,10 @@
 
 #include <stdbool.h>
 #include <stddef.h>
-#include <string.h>
 #include "color.h"
-#include "../util/macros.h"
-#include "../util/string-view.h"
-#include "../util/unicode.h"
+#include "util/macros.h"
+#include "util/string-view.h"
+#include "util/unicode.h"
 
 typedef struct {
     char buf[8192];
@@ -37,16 +36,24 @@ extern TermOutputBuffer obuf;
 
 #define term_add_literal(s) term_add_bytes(s, STRLEN(s))
 
+static inline size_t obuf_avail(void)
+{
+    return sizeof(obuf.buf) - obuf.count;
+}
+
 void term_output_reset(size_t start_x, size_t width, size_t scroll_x);
 void term_add_byte(char ch);
 void term_add_bytes(const char *str, size_t count);
 void term_set_bytes(char ch, size_t count);
 void term_repeat_byte(char ch, size_t count);
 void term_add_string_view(StringView sv);
-void term_sprintf(const char *fmt, ...) PRINTF(1);
 void term_add_str(const char *str);
+void term_add_uint(unsigned int x);
 void term_hide_cursor(void);
 void term_show_cursor(void);
+void term_move_cursor(unsigned int x, unsigned int y);
+void term_save_title(void);
+void term_restore_title(void);
 void term_clear_eol(void);
 void term_output_flush(void);
 bool term_put_char(CodePoint u);
