@@ -5,27 +5,29 @@
 #include <sys/types.h>
 #include "color.h"
 #include "key.h"
-#include "../util/macros.h"
-#include "../util/string-view.h"
+#include "util/macros.h"
+#include "util/string-view.h"
 
 typedef struct {
     StringView init;
     StringView deinit;
-    StringView reset_colors;
-    StringView reset_attrs;
     StringView keypad_off;
     StringView keypad_on;
     StringView cup_mode_off;
     StringView cup_mode_on;
     StringView show_cursor;
     StringView hide_cursor;
+    StringView save_title;
+    StringView restore_title;
+    StringView set_title_begin;
+    StringView set_title_end;
 } TermControlCodes;
 
 typedef struct {
     bool back_color_erase;
     TermColorCapabilityType color_type;
-    int width;
-    int height;
+    unsigned int width;
+    unsigned int height;
     unsigned int ncv_attributes;
     TermControlCodes control_codes;
     ssize_t (*parse_key_sequence)(const char *buf, size_t length, KeyCode *key);
@@ -33,20 +35,12 @@ typedef struct {
     void (*clear_screen)(void);
     void (*clear_to_eol)(void);
     void (*set_color)(const TermColor *color);
-    void (*move_cursor)(int x, int y);
+    void (*move_cursor)(unsigned int x, unsigned int y);
     void (*repeat_byte)(char ch, size_t count);
-    void (*raw)(void);
-    void (*cooked)(void);
-    void (*save_title)(void);
-    void (*restore_title)(void);
-    void (*set_title)(const char *title);
 } Terminal;
 
 extern Terminal terminal;
 
 void term_init(void);
 
-NORETURN COLD PRINTF(1)
-void term_init_fail(const char *fmt, ...);
-
 #endif
