@@ -6,12 +6,9 @@
 #include <stdint.h>
 #include "macros.h"
 
-#define HAS_GNUC5_OR_BUILTIN(b) (GNUC_AT_LEAST(5, 0) || HAS_BUILTIN(b))
-
-NONNULL_ARGS
 static inline bool size_add_overflows(size_t a, size_t b, size_t *result)
 {
-#if HAS_GNUC5_OR_BUILTIN(__builtin_add_overflow)
+#if HAS_BUILTIN(__builtin_add_overflow) || GNUC_AT_LEAST(5, 0)
     return __builtin_add_overflow(a, b, result);
 #else
     if (unlikely(b > SIZE_MAX - a)) {
@@ -22,10 +19,9 @@ static inline bool size_add_overflows(size_t a, size_t b, size_t *result)
 #endif
 }
 
-NONNULL_ARGS
 static inline bool size_multiply_overflows(size_t a, size_t b, size_t *result)
 {
-#if HAS_GNUC5_OR_BUILTIN(__builtin_mul_overflow)
+#if HAS_BUILTIN(__builtin_mul_overflow) || GNUC_AT_LEAST(5, 0)
     return __builtin_mul_overflow(a, b, result);
 #else
     if (unlikely(a > 0 && b > SIZE_MAX / a)) {
@@ -36,10 +32,9 @@ static inline bool size_multiply_overflows(size_t a, size_t b, size_t *result)
 #endif
 }
 
-NONNULL_ARGS
 static inline bool umax_add_overflows(uintmax_t a, uintmax_t b, uintmax_t *result)
 {
-#if HAS_GNUC5_OR_BUILTIN(__builtin_add_overflow)
+#if HAS_BUILTIN(__builtin_add_overflow) || GNUC_AT_LEAST(5, 0)
     return __builtin_add_overflow(a, b, result);
 #else
     if (unlikely(b > UINTMAX_MAX - a)) {
@@ -50,10 +45,9 @@ static inline bool umax_add_overflows(uintmax_t a, uintmax_t b, uintmax_t *resul
 #endif
 }
 
-NONNULL_ARGS
 static inline bool umax_multiply_overflows(uintmax_t a, uintmax_t b, uintmax_t *result)
 {
-#if HAS_GNUC5_OR_BUILTIN(__builtin_mul_overflow)
+#if HAS_BUILTIN(__builtin_mul_overflow) || GNUC_AT_LEAST(5, 0)
     return __builtin_mul_overflow(a, b, result);
 #else
     if (unlikely(a > 0 && b > UINTMAX_MAX / a)) {
